﻿using System.Linq;
using EmailClient.Applications;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using EmailClient.Applications.Views;
using EmailClient.Domain;
using EmailClient.Applications.ViewModels;
using EmailClient.Applications.Test.Mocks;
using System.Waf.UnitTesting;
using System.ComponentModel.Composition.Hosting;
using System.ComponentModel.Composition;
using EmailClient.Applications.Controllers;

namespace EmailClient.Applications.Test
{
    [TestClass]
    public class CreateEmailAccountControllerTest
    {
        public TestContext TestContext { get; set; }


        [TestMethod]
        public void CreateExchangeAccountTest()
        {
            // Create the IoC Container
            AggregateCatalog catalog = new AggregateCatalog();
            catalog.Catalogs.Add(new AssemblyCatalog(typeof(IApplicationController).Assembly));
            CompositionContainer container = new CompositionContainer(catalog);
            CompositionBatch batch = new CompositionBatch();
            batch.AddExportedValue(container);

            // Initialize the mock views
            MockShellView shellView = new MockShellView();
            MockEmailAccountsView emailAccountsView = new MockEmailAccountsView();
            MockCreateEmailAccountView createEmailAccountView = new MockCreateEmailAccountView();
            MockExchangeSettingsView exchangeSettingsView = new MockExchangeSettingsView();
            batch.AddExportedValue<IShellView>(shellView);
            batch.AddExportedValue<IMessageListView>(new MockMessageListView());
            batch.AddExportedValue<IMessageContentView>(new MockMessageContentView());
            batch.AddExportedValue<IEmailAccountsView>(emailAccountsView);
            batch.AddExportedValue<ICreateEmailAccountView>(createEmailAccountView);
            batch.AddExportedValue<IExchangeSettingsView>(exchangeSettingsView);
            container.Compose(batch);

            // Initialize the application controller and get the domain object
            IApplicationController applicationController = container.GetExportedValue<IApplicationController>();
            applicationController.Initialize();
            PrivateObject privateApplicationController = new PrivateObject(applicationController);
            EmailClientRoot emailClientRoot = (EmailClientRoot)privateApplicationController.GetField("emailClientRoot");

            // The emailClientRoot shouldn't contain any email accounts yet
            Assert.AreEqual(0, emailClientRoot.EmailAccounts.Count());

            // Run the application controller
            applicationController.Run();
            
            // The Shell should be visible now
            Assert.IsTrue(shellView.IsVisible);

            // Execute the Email Accounts command
            shellView.ViewModel.EmailAccountsCommand.Execute(null);

            // The wizard dialog should be visible now
            Assert.IsTrue(emailAccountsView.IsVisible);

            // Select the Exchange account and execute the next command
            createEmailAccountView.ViewModel.IsExchangeChecked = true;
            emailAccountsView.ViewModel.NextCommand.Execute(null);

            // Set the exchange account data and execute the next command
            exchangeSettingsView.ViewModel.Model.ServerPath = "exchange.test.com";
            exchangeSettingsView.ViewModel.Model.UserName = "testUser";
            emailAccountsView.ViewModel.NextCommand.Execute(null);

            // The wizard should be finished now and thus the dialog closed
            Assert.IsFalse(emailAccountsView.IsVisible);

            // Check that the email account is saved in our domain objects
            Assert.AreEqual(1, emailClientRoot.EmailAccounts.Count());
            ExchangeSettings exchangeSettings = (ExchangeSettings)emailClientRoot.EmailAccounts.ElementAt(0);
            Assert.AreEqual("exchange.test.com", exchangeSettings.ServerPath);
            Assert.AreEqual("testUser", exchangeSettings.UserName);

            // Close the Shell
            shellView.ViewModel.ExitCommand.Execute(null);
            Assert.IsFalse(shellView.IsVisible);
        }
    }
}
