﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using EmailClient.Applications.ViewModels;
using EmailClient.Applications.Test.Mocks;
using EmailClient.Domain;
using System.Waf.UnitTesting;

namespace EmailClient.Applications.Test
{
    [TestClass]
    public class Pop3SettingsViewModelTest
    {
        public TestContext TestContext { get; set; }


        [TestMethod]
        public void UseSameUserCreditsTest()
        {
            // Initialize the Pop3SettingsViewModel
            Pop3Settings pop3Settings = new Pop3Settings();
            MockPop3SettingsView view = new MockPop3SettingsView();
            Pop3SettingsViewModel viewModel = new Pop3SettingsViewModel(view, pop3Settings);

            // Set the userName and password for pop3 and smtp account
            viewModel.Model.Pop3UserCredits.UserName = "pop3.userName";
            viewModel.Model.Pop3UserCredits.Password = "pop3.password";
            viewModel.Model.SmtpUserCredits.UserName = "smtp.userName";
            viewModel.Model.SmtpUserCredits.Password = "smtp.password";

            // Activate the UseSameUserCredits feature
            viewModel.UseSameUserCredits = true;

            // Now the domain object must have the userName and password from the pop3 account
            Assert.AreEqual("pop3.userName", pop3Settings.SmtpUserCredits.UserName);
            Assert.AreEqual("pop3.password", pop3Settings.SmtpUserCredits.Password);

            // Check that changes of the pop3 userName are copied to the smtp userName as well
            viewModel.Model.Pop3UserCredits.UserName = "pop3.userName2";
            Assert.AreEqual("pop3.userName2", pop3Settings.SmtpUserCredits.UserName);

            // Check that changes of the pop3 password are copied to the smtp password as well
            viewModel.Model.Pop3UserCredits.Password = "pop3.password2";
            Assert.AreEqual("pop3.password2", pop3Settings.SmtpUserCredits.Password);
            
            // Deactivate the UseSameUserCredits feature
            viewModel.UseSameUserCredits = false;

            // Check that changes of the pop3 userName are not copied anymore to the smtp userName
            viewModel.Model.Pop3UserCredits.UserName = "pop3.userName3";
            Assert.AreEqual("pop3.userName2", pop3Settings.SmtpUserCredits.UserName);
        }
    }
}
