﻿using System;
using System.ComponentModel.Composition;
using System.ComponentModel.Composition.Hosting;
using System.Diagnostics;
using System.Linq;
using System.Waf.Applications;
using EmailClient.Applications.ViewModels;
using EmailClient.Applications.Views;
using EmailClient.Domain;

namespace EmailClient.Applications.Controllers
{
    /// <summary>
    /// Responsible for the application lifecycle.
    /// </summary>
    [Export(typeof(IApplicationController))]
    internal class ApplicationController : Controller, IApplicationController
    {
        private readonly CompositionContainer container;
        private readonly ShellViewModel shellViewModel;
        private readonly MessageListViewModel messageListViewModel;
        private readonly MessageContentViewModel messageContentViewModel;
        private readonly EmailClientRoot emailClientRoot;


        [ImportingConstructor]
        public ApplicationController(CompositionContainer container, ShellViewModel shellViewModel, 
            MessageListViewModel messageListViewModel, MessageContentViewModel messageContentViewModel)
        {
            if (container == null) { throw new ArgumentNullException("container"); }
            if (shellViewModel == null) { throw new ArgumentNullException("shellViewModel"); }
            if (messageListViewModel == null) { throw new ArgumentNullException("messageListViewModel"); }
            if (messageContentViewModel == null) { throw new ArgumentNullException("messageContentViewModel"); }
            
            this.container = container;
            this.shellViewModel = shellViewModel;
            this.messageListViewModel = messageListViewModel;
            this.messageContentViewModel = messageContentViewModel;
            this.emailClientRoot = new EmailClientRoot();
        }


        public void Initialize()
        {
            shellViewModel.ExitCommand = new DelegateCommand(Close);
            shellViewModel.EmailAccountsCommand = new DelegateCommand(RunCreateEmailAccountController);
            shellViewModel.MessageListView = messageListViewModel.View;
            shellViewModel.MessageContentView = messageContentViewModel.View;
        }

        public void Run()
        {
            shellViewModel.Show();
        }

        public void Shutdown()
        {
        }

        private void Close()
        {
            shellViewModel.Close();
        }

        private void RunCreateEmailAccountController()
        {
            CreateEmailAccountController useCaseController = new CreateEmailAccountController(container, 
                shellViewModel.View, emailClientRoot);
            useCaseController.Initialize();
            useCaseController.Run();

            // Just for debugging
            Trace.WriteLine(string.Format(null, "\n> Current email accounts ({0}):", 
                emailClientRoot.EmailAccounts.Count()));
            emailClientRoot.EmailAccounts.ToList().ForEach(x => Trace.WriteLine(x));
        }
    }
}
