﻿using System;
using System.ComponentModel;
using System.ComponentModel.Composition;
using System.Waf.Applications;
using EmailClient.Applications.Views;
using EmailClient.Domain;

namespace EmailClient.Applications.ViewModels
{
    [Export, PartCreationPolicy(CreationPolicy.NonShared)]
    public class Pop3SettingsViewModel : ViewModel<IPop3SettingsView>
    {
        private readonly Pop3Settings model;
        private bool useSameUserCredits;


        [ImportingConstructor]
        public Pop3SettingsViewModel(IPop3SettingsView view, Pop3Settings pop3Settings) : base(view)
        {
            if (pop3Settings == null) { throw new ArgumentNullException("pop3Settings"); }
            
            this.model = pop3Settings;
            AddWeakEventListener(this.model.Pop3UserCredits, Pop3UserCreditsPropertyChanged);
        }


        public Pop3Settings Model { get { return model; } }

        public bool UseSameUserCredits
        {
            get { return useSameUserCredits; }
            set 
            {
                if (useSameUserCredits != value)
                {
                    useSameUserCredits = value;
                    if (useSameUserCredits)
                    {
                        Model.SmtpUserCredits.UserName = Model.Pop3UserCredits.UserName;
                        Model.SmtpUserCredits.Password = Model.Pop3UserCredits.Password;
                    }
                    RaisePropertyChanged("UseSameUserCredits");
                }
            }
        }

        private void Pop3UserCreditsPropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            if (e.PropertyName == "UserName")
            {
                if (UseSameUserCredits)
                {
                    Model.SmtpUserCredits.UserName = Model.Pop3UserCredits.UserName;
                }
            }
            else if (e.PropertyName == "Password")
            {
                if (UseSameUserCredits)
                {
                    Model.SmtpUserCredits.Password = Model.Pop3UserCredits.Password;
                }
            }
        }
    }
}
