﻿using System.ComponentModel.Composition;
using System.ComponentModel.Composition.Hosting;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Writer.Applications.Controllers;
using Writer.Applications.Documents;
using Writer.Applications.Test.Services;
using Writer.Applications.Test.Views;
using Writer.Applications.Services;
using Writer.Applications.ViewModels;

namespace Writer.Applications.Test
{
    [TestClass]
    public abstract class TestClassBase
    {
        private readonly CompositionContainer container;
        private PrintController printController;
        private RichTextDocumentController richTextDocumentController;

        
        protected TestClassBase()
        {
            AggregateCatalog catalog = new AggregateCatalog();
            catalog.Catalogs.Add(new TypeCatalog(
                typeof(ApplicationController), typeof(PrintController), typeof(FileController), typeof(RichTextDocumentController),
                typeof(ShellService), typeof(FileService), typeof(MockEnvironmentService),
                typeof(MainViewModel), typeof(ShellViewModel), typeof(StartViewModel)
            ));
            catalog.Catalogs.Add(new TypeCatalog(
                typeof(MockPresentationService),
                typeof(MockMessageService), typeof(MockFileDialogService), typeof(MockPrintDialogService),
                typeof(MockShellView), typeof(MockMainView), typeof(MockRichTextView), typeof(MockSaveChangesView),
                typeof(MockPrintPreviewView), typeof(MockStartView)
            ));
            container = new CompositionContainer(catalog);
            CompositionBatch batch = new CompositionBatch();
            batch.AddExportedValue(container);
            container.Compose(batch);
        }


        protected CompositionContainer Container { get { return container; } }


        [TestInitialize]
        public void TestInitialize()
        {
            richTextDocumentController = Container.GetExportedValue<RichTextDocumentController>();
            Container.GetExportedValue<FileController>().Initialize();
            
            OnTestInitialize();
        }

        [TestCleanup]
        public void TestCleanup()
        {
            OnTestCleanup();
        }

        protected virtual void OnTestInitialize() { }

        protected virtual void OnTestCleanup() { }

        internal PrintController InitializePrintController()
        {
            printController = container.GetExportedValue<PrintController>();
            printController.Initialize();
            return printController;
        }

        protected void ShutdownPrintController()
        {
            printController.Shutdown();
            printController = null;
        }
    }
}
